@@ -92,14 +92,24 @@ class RequestGroupSearchContext(object):
         # or a sharing provider.
         self.use_same_provider = group.use_same_provider
 
-        # maps the trait name to the trait internal ID
-        self.required_trait_map = {}
-        self.forbidden_trait_map = {}
-        for trait_map, traits in (
-                (self.required_trait_map, group.required_traits),
-                (self.forbidden_trait_map, group.forbidden_traits)):
-            if traits:
-                trait_map.update(trait_obj.ids_from_names(context, traits))
+        # Both required_trait_names and required_traits expresses the same
+        # request with the same nested list of sets structure but
+        # required_trait_names contains trait names while required_traits
+        # contains trait internal IDs
+        self.required_trait_names = group.required_traits
+        # let's map the trait names to internal IDs this is useful for DB calls
+        # expecting trait IDs. The structure of this field is the same as the
+        # required_trait_names field.
+        self.required_traits = []
+        # forbidden_traits is a dict mapping trait names to trait internal IDs
+        self.forbidden_traits = {}
+
+        for any_traits in group.required_traits:
+            self.required_traits.append(
+                set(trait_obj.ids_from_names(context, any_traits).values()))
+        if group.forbidden_traits:
+            self.forbidden_traits = trait_obj.ids_from_names(
+                context, group.forbidden_traits)
 
         # Internal id of a root provider. If provided, this RequestGroup must
         # be satisfied by resource provider(s) under the root provider.
@@ -397,7 +407,7 @@ def provider_ids_from_uuid(context, uuid):
     me_to_parent = sa.outerjoin(
         me_to_root, parent,
         me.c.parent_provider_id == parent.c.id)
-    sel = sa.select(cols).select_from(me_to_parent)
+    sel = sa.select(*cols).select_from(me_to_parent)
     sel = sel.where(me.c.uuid == uuid)
     res = context.session.execute(sel).fetchone()
     if not res:
@@ -406,13 +416,17 @@ def provider_ids_from_uuid(context, uuid):
 
 
 def _usage_select(rc_ids):
-    usage = sa.select([_ALLOC_TBL.c.resource_provider_id,
-                       _ALLOC_TBL.c.resource_class_id,
-                       sql.func.sum(_ALLOC_TBL.c.used).label('used')])
-    usage = usage.where(_ALLOC_TBL.c.resource_class_id.in_(rc_ids))
-    usage = usage.group_by(_ALLOC_TBL.c.resource_provider_id,
-                           _ALLOC_TBL.c.resource_class_id)
-    return sa.alias(usage, name='usage')
+    usage = sa.select(
+        _ALLOC_TBL.c.resource_provider_id,
+        _ALLOC_TBL.c.resource_class_id,
+        sql.func.sum(_ALLOC_TBL.c.used).label('used')
+    ).where(
+        _ALLOC_TBL.c.resource_class_id.in_(rc_ids)
+    ).group_by(
+        _ALLOC_TBL.c.resource_provider_id,
+        _ALLOC_TBL.c.resource_class_id,
+    )
+    return usage.subquery(name='usage')
 
 
 def _capacity_check_clause(amount, usage, inv_tbl=_INV_TBL):
@@ -469,7 +483,7 @@ def get_providers_with_resource(ctx, rc_id, amount, tree_root_id=None):
     inv_to_usage = sa.outerjoin(
         rp_to_inv, usage,
         inv.c.resource_provider_id == usage.c.resource_provider_id)
-    sel = sa.select([rpt.c.id, rpt.c.root_provider_id])
+    sel = sa.select(rpt.c.id, rpt.c.root_provider_id)
     sel = sel.select_from(inv_to_usage)
     where_conds = _capacity_check_clause(amount, usage, inv_tbl=inv)
     if tree_root_id is not None:
@@ -495,7 +509,7 @@ def get_providers_with_root(ctx, allowed, forbidden):
     # FROM resource_providers AS rp
     # WHERE rp.id IN ($allowed)
     # AND rp.id NOT IN ($forbidden)
-    sel = sa.select([_RP_TBL.c.id, _RP_TBL.c.root_provider_id])
+    sel = sa.select(_RP_TBL.c.id, _RP_TBL.c.root_provider_id)
     sel = sel.select_from(_RP_TBL)
     cond = []
     if allowed:
@@ -716,7 +730,7 @@ def get_trees_matching_all(rg_ctx, rw_ctx):
         if not provs_with_inv:
             return rp_candidates.RPCandidateList()
 
-    if (not rg_ctx.required_trait_map and not rg_ctx.forbidden_trait_map) or (
+    if (not rg_ctx.required_traits and not rg_ctx.forbidden_traits) or (
             rg_ctx.exists_sharing):
         # If there were no traits required, there's no difference in how we
         # calculate allocation requests between nested and non-nested
@@ -729,14 +743,14 @@ def get_trees_matching_all(rg_ctx, rw_ctx):
     # capacity and that set of providers (grouped by their tree) have all
     # of the required traits and none of the forbidden traits
     rp_tuples_with_trait = _get_trees_with_traits(
-        rg_ctx.context, provs_with_inv.rps, rg_ctx.required_trait_map,
-        rg_ctx.forbidden_trait_map)
+        rg_ctx.context, provs_with_inv.rps, rg_ctx.required_traits,
+        rg_ctx.forbidden_traits)
     provs_with_inv.filter_by_rp(rp_tuples_with_trait)
     LOG.debug("found %d providers under %d trees after applying "
               "traits filter - required: %s, forbidden: %s",
               len(provs_with_inv.rps), len(provs_with_inv.trees),
-              list(rg_ctx.required_trait_map),
-              list(rg_ctx.forbidden_trait_map))
+              list(rg_ctx.required_trait_names),
+              list(rg_ctx.forbidden_traits))
 
     return provs_with_inv
 
@@ -747,87 +761,119 @@ def _get_trees_with_traits(ctx, rp_ids, required_traits, forbidden_traits):
     (provider ID, root provider ID) of providers which belong to a tree that
     can satisfy trait requirements.
 
+    This returns trees that still have the possibility to be a match according
+    to the required and forbidden traits. It returns every rp from
+    the tree that is in rp_ids, even if some of those rps are providing
+    forbidden traits.
+    This filters out a whole tree if either:
+    * every RPs of the tree from rp_ids having a forbidden trait (see
+      test_get_trees_with_traits_forbidden_1 and _2)
+    * there is a required trait that none of the RPs of the tree from rp_ids
+      provide (see test_get_trees_with_traits) or there is an RP providing
+      the required trait but that also provides a forbidden trait
+      (see test_get_trees_with_traits_forbidden_3)
+
+    The returned tree still might not be a valid tree as this function
+    returns a tree even if some providers need to be ignored due to forbidden
+    traits. So if those RPs are needed from resource perspective then the tree
+    will be filtered out later by
+    objects.allocation_candidate._check_traits_for_alloc_request
+
     :param ctx: Session context to use
     :param rp_ids: a set of resource provider IDs
-    :param required_traits: A map, keyed by trait string name, of required
-                            trait internal IDs that each provider TREE must
-                            COLLECTIVELY have associated with it
-    :param forbidden_traits: A map, keyed by trait string name, of trait
-                             internal IDs that a resource provider must
-                             not have.
+    :param required_traits: A list of set of trait internal IDs where the
+       traits in each nested set are OR'd while the items in the outer list are
+       AND'd together. The RPs in the tree should COLLECTIVELY fulfill this
+       trait request.
+    :param forbidden_traits: A list of trait internal IDs that a resource
+        provider tree must not have.
     """
-    # We now want to restrict the returned providers to only those provider
-    # trees that have all our required traits.
-    #
-    # The SQL we want looks like this:
-    #
-    # SELECT outer_rp.id, outer_rp.root_provider_id
-    # FROM resource_providers AS outer_rp
-    # JOIN (
-    #   SELECT rp.root_provider_id
-    #   FROM resource_providers AS rp
-    #   # Only if we have required traits...
-    #   INNER JOIN resource_provider_traits AS rptt
-    #   ON rp.id = rptt.resource_provider_id
-    #   AND rptt.trait_id IN ($REQUIRED_TRAIT_IDS)
-    #   # Only if we have forbidden_traits...
-    #   LEFT JOIN resource_provider_traits AS rptt_forbid
-    #   ON rp.id = rptt_forbid.resource_provider_id
-    #   AND rptt_forbid.trait_id IN ($FORBIDDEN_TRAIT_IDS)
-    #   WHERE rp.id IN ($RP_IDS)
-    #   # Only if we have forbidden traits...
-    #   AND rptt_forbid.resource_provider_id IS NULL
-    #   GROUP BY rp.root_provider_id
-    #   # Only if have required traits...
-    #   HAVING COUNT(DISTINCT rptt.trait_id) == $NUM_REQUIRED_TRAITS
-    # ) AS trees_with_traits
-    #  ON outer_rp.root_provider_id = trees_with_traits.root_provider_id
-    rpt = sa.alias(_RP_TBL, name="rp")
-    cond = [rpt.c.id.in_(rp_ids)]
-    subq = sa.select([rpt.c.root_provider_id])
-    subq_join = None
-    if required_traits:
-        rptt = sa.alias(_RP_TRAIT_TBL, name="rptt")
-        rpt_to_rptt = sa.join(
-            rpt, rptt, sa.and_(
-                rpt.c.id == rptt.c.resource_provider_id,
-                rptt.c.trait_id.in_(required_traits.values())))
-        subq_join = rpt_to_rptt
-        # Only get the resource providers that have ALL the required traits,
-        # so we need to GROUP BY the root provider and ensure that the
-        # COUNT(trait_id) is equal to the number of traits we are requiring
-        num_traits = len(required_traits)
-        having_cond = sa.func.count(sa.distinct(rptt.c.trait_id)) == num_traits
-        subq = subq.having(having_cond)
+    # TODO(gibi): if somebody can formulate the below three SQL query to a
+    # single one then probably that will improve performance
 
-    # Tack on an additional LEFT JOIN clause inside the derived table if we've
-    # got forbidden traits in the mix.
-    if forbidden_traits:
-        rptt_forbid = sa.alias(_RP_TRAIT_TBL, name="rptt_forbid")
-        join_to = rpt
-        if subq_join is not None:
-            join_to = subq_join
-        rpt_to_rptt_forbid = sa.outerjoin(
-            join_to, rptt_forbid, sa.and_(
-                rpt.c.id == rptt_forbid.c.resource_provider_id,
-                rptt_forbid.c.trait_id.in_(forbidden_traits.values())))
-        cond.append(rptt_forbid.c.resource_provider_id == sa.null())
-        subq_join = rpt_to_rptt_forbid
+    # Get the root of all rps in the rp_ids as we need to return every rp from
+    # rp_ids that is in a matching tree but below we will filter out rps by
+    # traits. So we need a copy and also that copy needs to associate rps to
+    # trees by root_id
+    rpt = sa.alias(_RP_TBL, name='rpt')
+    sel = sa.select(rpt.c.id, rpt.c.root_provider_id).select_from(rpt)
+    sel = sel.where(rpt.c.id.in_(rp_ids))
+    res = ctx.session.execute(sel).fetchall()
+    original_rp_ids = {rp_id: root_id for rp_id, root_id in res}
 
-    subq = subq.select_from(subq_join)
-    subq = subq.where(sa.and_(*cond))
-    subq = subq.group_by(rpt.c.root_provider_id)
-    trees_with_traits = sa.alias(subq, name="trees_with_traits")
-
-    outer_rps = sa.alias(_RP_TBL, name="outer_rps")
-    outer_to_subq = sa.join(
-        outer_rps, trees_with_traits,
-        outer_rps.c.root_provider_id == trees_with_traits.c.root_provider_id)
-    sel = sa.select([outer_rps.c.id, outer_rps.c.root_provider_id])
-    sel = sel.select_from(outer_to_subq)
+    # First filter out the rps from the rp_ids list that provide forbidden
+    # traits. To do that we collect those rps that provide any of the forbidden
+    # traits and with the outer join and the null check we filter them out
+    # of the result
+    rptt_forbidden = sa.alias(_RP_TRAIT_TBL, name="rptt_forbidden")
+    rp_to_trait = sa.outerjoin(
+        rpt, rptt_forbidden,
+        sa.and_(
+            rpt.c.id == rptt_forbidden.c.resource_provider_id,
+            rptt_forbidden.c.trait_id.in_(forbidden_traits)
+        )
+    )
+    sel = sa.select(rpt.c.id, rpt.c.root_provider_id).select_from(rp_to_trait)
+    sel = sel.where(
+        sa.and_(
+            rpt.c.id.in_(original_rp_ids.keys()),
+            rptt_forbidden.c.trait_id == sa.null()
+        )
+    )
     res = ctx.session.execute(sel).fetchall()
 
-    return set((rp_id, root_id) for rp_id, root_id in res)
+    # These are the rps that does not provide any forbidden traits
+    good_rp_ids = {}
+    for rp_id, root_id in res:
+        good_rp_ids[rp_id] = root_id
+
+    # shortcut if no traits required the good_rp_ids.values() contains all the
+    # good roots
+    if not required_traits:
+        return {
+            (rp_id, root_id)
+            for rp_id, root_id in original_rp_ids.items()
+            if root_id in good_rp_ids.values()
+        }
+
+    # now get the traits provided by the good rps per tree
+    rptt = sa.alias(_RP_TRAIT_TBL, name="rptt")
+    rp_to_trait = sa.join(
+        rpt, rptt, rpt.c.id == rptt.c.resource_provider_id)
+    sel = sa.select(
+        rpt.c.root_provider_id, rptt.c.trait_id
+    ).select_from(rp_to_trait)
+    sel = sel.where(rpt.c.id.in_(good_rp_ids))
+    res = ctx.session.execute(sel).fetchall()
+
+    root_to_traits = collections.defaultdict(set)
+    for root_id, trait_id in res:
+        root_to_traits[root_id].add(trait_id)
+
+    result = set()
+
+    # filter the trees by checking if each tree provides all the
+    # required_traits
+    for root_id, provided_traits in root_to_traits.items():
+        # we need a match for all the items from the outer list of the
+        # required_traits as that describes AND relationship
+        if all(
+            # we need at least one match per nested trait set as that set
+            # describes OR relationship
+            any_traits.intersection(provided_traits)
+            for any_traits in required_traits
+        ):
+            # This tree is matching the required traits so add result all the
+            # rps from the original rp_ids that belongs to this tree
+            result.update(
+                {
+                    (rp_id, root_id)
+                    for rp_id, original_root_id in original_rp_ids.items()
+                    if root_id == original_root_id
+                }
+
+            )
+    return result
 
 
 @db_api.placement_context_manager.reader
@@ -855,7 +901,7 @@ def _get_roots_with_traits(ctx, required_traits, forbidden_traits):
     #
     #   SELECT rp.id FROM resource_providers AS rp
     rpt = sa.alias(_RP_TBL, name="rp")
-    sel = sa.select([rpt.c.id])
+    sel = sa.select(rpt.c.id)
 
     #   WHERE rp.parent_provider_id IS NULL
     cond = [rpt.c.parent_provider_id.is_(None)]
@@ -959,7 +1005,7 @@ def provider_ids_matching_aggregates(context, member_of, rp_ids=None):
         for member in members:
             agg_uuids.add(member)
     agg_tbl = sa.alias(_AGG_TBL, name='aggs')
-    agg_sel = sa.select([agg_tbl.c.uuid, agg_tbl.c.id])
+    agg_sel = sa.select(agg_tbl.c.uuid, agg_tbl.c.id)
     agg_sel = agg_sel.where(agg_tbl.c.uuid.in_(agg_uuids))
     agg_uuid_map = {
         r[0]: r[1] for r in context.session.execute(agg_sel).fetchall()
@@ -982,7 +1028,79 @@ def provider_ids_matching_aggregates(context, member_of, rp_ids=None):
             rp_tbl.c.id == rpa_tbl.c.resource_provider_id,
             rpa_tbl.c.aggregate_id.in_(agg_ids))
         join_chain = sa.join(join_chain, rpa_tbl, join_cond)
-    sel = sa.select([rp_tbl.c.id]).select_from(join_chain)
+    sel = sa.select(rp_tbl.c.id).select_from(join_chain)
+    if rp_ids:
+        sel = sel.where(rp_tbl.c.id.in_(rp_ids))
+    return set(r[0] for r in context.session.execute(sel))
+
+
+@db_api.placement_context_manager.reader
+def provider_ids_matching_required_traits(
+    context, required_traits, rp_ids=None
+):
+    """Given a list of set of trait internal IDs, return the internal IDs of
+    all resource providers that individually satisfy the requested traits.
+
+    :param context: The request context
+    :param required_traits: A non-empty list containing sets of trait IDs.
+        Each item in the outer list is to be AND'd together. If that item
+        contains multiple values, they are OR'd together.
+
+        For example, if required is::
+
+            [
+                {'trait1ID'},
+                {'trait2ID', 'trait3ID'},
+            ]
+
+        we will return all the resource providers that has trait1 and either
+        trait2 or trait3.
+     :param rp_ids: When present, returned resource providers are limited
+        to only those in this value
+
+    :returns: A set of internal resource provider IDs having all required
+        traits
+    """
+    if not required_traits:
+        raise ValueError('required_traits must not be empty')
+
+    # Given a request for the following:
+    #
+    # required = [
+    #   {trait1},
+    #   {trait2},
+    #   {trait3, trait4}
+    # ]
+    #
+    # we need to produce the following SQL expression:
+    #
+    # SELECT
+    #   rp.id
+    # FROM resource_providers AS rp
+    # JOIN resource_provider_traits AS rpt1
+    #   ON rp.id = rpt1.resource_provider_id
+    #   AND rpt1.trait_id IN ($TRAIT1_ID)
+    # JOIN resource_provider_traits AS rpt2
+    #   ON rp.id = rpt2.resource_provider_id
+    #   AND rpt2.trait_id IN ($TRAIT2_ID)
+    # JOIN resource_provider_traits AS rpt3
+    #   ON rp.id = rpt3.resource_provider_id
+    #   AND rpt3.trait_id IN ($TRAIT3_ID, $TRAIT4_ID)
+    # # Only if we have rp_ids...
+    # WHERE rp.id IN ($RP_IDs)
+
+    rp_tbl = sa.alias(_RP_TBL, name='rp')
+    join_chain = rp_tbl
+
+    for x, any_traits in enumerate(required_traits):
+        rpt_tbl = sa.alias(_RP_TRAIT_TBL, name='rpt%d' % x)
+
+        join_cond = sa.and_(
+            rp_tbl.c.id == rpt_tbl.c.resource_provider_id,
+            rpt_tbl.c.trait_id.in_(any_traits))
+        join_chain = sa.join(join_chain, rpt_tbl, join_cond)
+
+    sel = sa.select(rp_tbl.c.id).select_from(join_chain)
     if rp_ids:
         sel = sel.where(rp_tbl.c.id.in_(rp_ids))
     return set(r[0] for r in context.session.execute(sel))
@@ -990,54 +1108,24 @@ def provider_ids_matching_aggregates(context, member_of, rp_ids=None):
 
 @db_api.placement_context_manager.reader
 def get_provider_ids_having_any_trait(ctx, traits):
-    """Returns a set of resource provider internal IDs that have ANY of the
-    supplied traits.
+    """Returns a set of resource provider internal IDs that individually
+    have ANY of the supplied traits.
 
     :param ctx: Session context to use
-    :param traits: A map, keyed by trait string name, of trait internal IDs, at
-                   least one of which each provider must have associated with
-                   it.
+    :param traits: An iterable of trait internal IDs, at least one of which
+        each provider must have associated with it.
     :raise ValueError: If traits is empty or None.
     """
     if not traits:
         raise ValueError('traits must not be empty')
 
     rptt = sa.alias(_RP_TRAIT_TBL, name="rpt")
-    sel = sa.select([rptt.c.resource_provider_id])
-    sel = sel.where(rptt.c.trait_id.in_(traits.values()))
+    sel = sa.select(rptt.c.resource_provider_id)
+    sel = sel.where(rptt.c.trait_id.in_(traits))
     sel = sel.group_by(rptt.c.resource_provider_id)
     return set(r[0] for r in ctx.session.execute(sel))
 
 
-@db_api.placement_context_manager.reader
-def _get_provider_ids_having_all_traits(ctx, required_traits):
-    """Returns a set of resource provider internal IDs that have ALL of the
-    required traits.
-
-    NOTE: Don't call this method with no required_traits.
-
-    :param ctx: Session context to use
-    :param required_traits: A map, keyed by trait string name, of required
-                            trait internal IDs that each provider must have
-                            associated with it
-    :raise ValueError: If required_traits is empty or None.
-    """
-    if not required_traits:
-        raise ValueError('required_traits must not be empty')
-
-    rptt = sa.alias(_RP_TRAIT_TBL, name="rpt")
-    sel = sa.select([rptt.c.resource_provider_id])
-    sel = sel.where(rptt.c.trait_id.in_(required_traits.values()))
-    sel = sel.group_by(rptt.c.resource_provider_id)
-    # Only get the resource providers that have ALL the required traits, so we
-    # need to GROUP BY the resource provider and ensure that the
-    # COUNT(trait_id) is equal to the number of traits we are requiring
-    num_traits = len(required_traits)
-    cond = sa.func.count(rptt.c.trait_id) == num_traits
-    sel = sel.having(cond)
-    return set(r[0] for r in ctx.session.execute(sel))
-
-
 def get_provider_ids_for_traits_and_aggs(rg_ctx):
     """Get internal IDs for all providers matching the specified traits/aggs.
 
@@ -1050,13 +1138,13 @@ def get_provider_ids_for_traits_and_aggs(rg_ctx):
             specified forbidden_traits.
     """
     filtered_rps = set()
-    if rg_ctx.required_trait_map:
-        trait_rps = _get_provider_ids_having_all_traits(
-            rg_ctx.context, rg_ctx.required_trait_map)
+    if rg_ctx.required_traits:
+        trait_rps = provider_ids_matching_required_traits(
+            rg_ctx.context, rg_ctx.required_traits)
         filtered_rps = trait_rps
         LOG.debug("found %d providers after applying required traits filter "
                   "(%s)",
-                  len(filtered_rps), list(rg_ctx.required_trait_map))
+                  len(filtered_rps), list(rg_ctx.required_trait_names))
         if not filtered_rps:
             return None, []
 
@@ -1084,15 +1172,15 @@ def get_provider_ids_for_traits_and_aggs(rg_ctx):
             if not filtered_rps:
                 return None, []
 
-    if rg_ctx.forbidden_trait_map:
+    if rg_ctx.forbidden_traits:
         rps_bad_traits = get_provider_ids_having_any_trait(
-            rg_ctx.context, rg_ctx.forbidden_trait_map)
+            rg_ctx.context, rg_ctx.forbidden_traits.values())
         forbidden_rp_ids |= rps_bad_traits
         if filtered_rps:
             filtered_rps -= rps_bad_traits
             LOG.debug("found %d providers after applying forbidden traits "
                       "filter (%s)", len(filtered_rps),
-                      list(rg_ctx.forbidden_trait_map))
+                      list(rg_ctx.forbidden_traits))
             if not filtered_rps:
                 return None, []
 
@@ -1155,7 +1243,7 @@ def get_sharing_providers(ctx, rp_ids=None):
                 rpt_tbl.c.trait_id == sharing_trait.id)
     )
 
-    sel = sa.select([rp_tbl.c.id]).select_from(rp_to_rpt_join)
+    sel = sa.select(rp_tbl.c.id).select_from(rp_to_rpt_join)
     if rp_ids:
         sel = sel.where(rp_tbl.c.id.in_(rp_ids))
 
@@ -1201,7 +1289,7 @@ def anchors_for_sharing_providers(context, rp_ids):
         shr_with_sps_aggs.c.resource_provider_id == shr_with_sps.c.id)
     join_chain = sa.join(
         join_chain, rps, shr_with_sps.c.root_provider_id == rps.c.id)
-    sel = sa.select([sps.c.id, sps.c.uuid, rps.c.id, rps.c.uuid])
+    sel = sa.select(sps.c.id, sps.c.uuid, rps.c.id, rps.c.uuid)
     sel = sel.select_from(join_chain)
     sel = sel.where(sps.c.id.in_(rp_ids))
     return set([
@@ -1218,7 +1306,7 @@ def _has_provider_trees(ctx):
 
     NOTE(jaypipes): The result of this function can be cached extensively.
     """
-    sel = sa.select([_RP_TBL.c.id])
+    sel = sa.select(_RP_TBL.c.id)
     sel = sel.where(_RP_TBL.c.parent_provider_id.isnot(None))
     sel = sel.limit(1)
     res = ctx.session.execute(sel).fetchall()
@@ -1265,16 +1353,14 @@ def get_usages_by_provider_trees(ctx, root_ids):
                 _RP_TBL.c.root_provider_id.in_(sa.bindparam(
                     'root_ids', expanding=True)))
     )
-    usage = sa.alias(
-        sa.select([
-            _ALLOC_TBL.c.resource_provider_id,
-            _ALLOC_TBL.c.resource_class_id,
-            sql.func.sum(_ALLOC_TBL.c.used).label('used'),
-        ]).select_from(derived_alloc_to_rp).group_by(
-            _ALLOC_TBL.c.resource_provider_id,
-            _ALLOC_TBL.c.resource_class_id
-        ),
-        name='usage')
+    usage = sa.select(
+        _ALLOC_TBL.c.resource_provider_id,
+        _ALLOC_TBL.c.resource_class_id,
+        sql.func.sum(_ALLOC_TBL.c.used).label('used'),
+    ).select_from(derived_alloc_to_rp).group_by(
+        _ALLOC_TBL.c.resource_provider_id,
+        _ALLOC_TBL.c.resource_class_id
+    ).subquery(name='usage')
     # Build a join between the resource providers and inventories table
     rpt_inv_join = sa.outerjoin(rpt, inv,
                                 rpt.c.id == inv.c.resource_provider_id)
@@ -1287,7 +1373,7 @@ def get_usages_by_provider_trees(ctx, root_ids):
             usage.c.resource_class_id == inv.c.resource_class_id,
         ),
     )
-    query = sa.select([
+    query = sa.select(
         rpt.c.id.label("resource_provider_id"),
         rpt.c.uuid.label("resource_provider_uuid"),
         inv.c.resource_class_id,
@@ -1296,7 +1382,7 @@ def get_usages_by_provider_trees(ctx, root_ids):
         inv.c.allocation_ratio,
         inv.c.max_unit,
         usage.c.used,
-    ]).select_from(usage_join).where(
+    ).select_from(usage_join).where(
         rpt.c.root_provider_id.in_(sa.bindparam(
             'root_ids', expanding=True))
     )
